/*
 * Copyright (C) 2005 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License.
 * You must obey the GNU General Public License in all respects for all of the code used
 * other than the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.entity;

import java.util.List;

import mir.storage.Database;
import mir.storage.DatabaseFailure;
import mir.storage.DatabaseExc;
import mir.util.RewindableIterator;
import multex.Failure;

public class EntityBrowser implements RewindableIterator {

  private Database database;
  private String mainTablePrefix;
  private List extraTables;
  private String whereClause;
  private String orderByClause;
  private int batchSize;
  private EntityList currentBatch;

  private int skip;
  private int limit;

  private int batchPosition;
  private int positionInBatch;

  public EntityBrowser(Database aDatabase, String aMainTablePrefix, List someExtraTables,
     String aWhereClause, String anOrderByClause,
     int aBatchSize, int aLimit, int aSkip) throws DatabaseFailure {

    database=aDatabase;
    mainTablePrefix=aMainTablePrefix;
    extraTables=someExtraTables;
    whereClause=aWhereClause;
    orderByClause=anOrderByClause;
    batchSize=aBatchSize;
    skip=aSkip;
    limit=aLimit;

    rewind();
  }

  public EntityBrowser(Database aDatabase, String aWhereClause, String anOrderByClause,
                       int aBatchSize, int aLimit, int aSkip) throws DatabaseFailure {
    this(aDatabase, "", null, aWhereClause, anOrderByClause, aBatchSize, aLimit, aSkip);
  }

  public EntityBrowser(Database aDatabase,
          String aWhereClause, String anOrderByClause,
          int aBatchSize) throws DatabaseFailure {
    this(aDatabase, aWhereClause, anOrderByClause, aBatchSize, -1, 0);
  }

  public void readCurrentBatch(int aSkip) throws DatabaseExc, DatabaseFailure {
    currentBatch = database.selectByWhereClause(mainTablePrefix, extraTables,
        whereClause, orderByClause, aSkip, batchSize);
    batchPosition = aSkip;
    positionInBatch = 0;
  }

  public void rewind() {
    try {
      readCurrentBatch(skip);
    }
    catch (Throwable t) {
      throw new Failure("Error while rewinding", t);
    }
  }

  public boolean hasNext() {
    try {
      if (limit>-1 && batchPosition+positionInBatch>=skip+limit)
        return false;

      if (positionInBatch>=currentBatch.size() && currentBatch.hasNextBatch()) {
        readCurrentBatch(batchPosition+positionInBatch);
      }

      return (positionInBatch<currentBatch.size());
    }
    catch (Throwable t) {
      throw new RuntimeException(t.getMessage());
    }
  }

  public Object next() {
    try {
      if (hasNext()) {
        Entity result = currentBatch.elementAt(positionInBatch);
        positionInBatch=positionInBatch+1;

        return result;
      }
			return null;
    }
    catch (Throwable t) {
      throw new RuntimeException(t.getMessage());
    }
  }

  public void remove() {
    throw new UnsupportedOperationException();
  }
}