/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mircoders.localizer.basic;

import mir.config.MirPropertiesConfiguration;
import mir.generator.WriterEngine;
import mir.log.LoggerWrapper;
import mir.util.FileRoutines;
import mircoders.localizer.MirLocalizerFailure;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.Writer;

public class MirBasicWriterEngine implements WriterEngine {
  private String defaultEncoding;

  protected static LoggerWrapper logger = new LoggerWrapper("Localizer.WriterEngine");
  protected MirPropertiesConfiguration configuration = MirPropertiesConfiguration.instance();

  /**
   * Directory to store temp files into
   */
  private File tempDirectory;

  public MirBasicWriterEngine(String aDefaultEncoding) {
    defaultEncoding = aDefaultEncoding;
    tempDirectory = configuration.getFile("TempDir");
  }

  /**
   * {@inheritDoc}
   */
  public Object openWriter(String anIdentifier, String anEncoding) throws MirLocalizerFailure {
    String encoding;

    if (anEncoding != null && !anEncoding.equals("")) {
      encoding = anEncoding;
    }
    else {
      encoding = defaultEncoding;
    }

    File destinationFile = new File(anIdentifier);

    File destinationDirectory  = destinationFile.getParentFile();

    if (destinationDirectory != null && !destinationDirectory.exists()) {
      destinationDirectory.mkdirs();
    }

    try {
      return new PrintWriter(
          new TempWriter(destinationFile, encoding)
      );
    }
    catch (IOException t) {
      throw new MirLocalizerFailure("Failure while opening a writer", t);
    }
  }

  public void closeWriter(Object aWriter) {
    ((PrintWriter) aWriter).close();
  }

  private class TempWriter extends Writer {
    TempWriter(File aDestination, String anEncoding) throws IOException {
      destinationFile = aDestination;
      slaveFile = File.createTempFile("Mir", ".generated", tempDirectory);
      slaveFile.deleteOnExit();
      slaveWriter = new BufferedWriter(
          new OutputStreamWriter(
            new FileOutputStream(slaveFile), anEncoding
      ), 8192);
    }

    public void close() throws IOException {
      slaveWriter.close();

      try {
        FileRoutines.move(slaveFile, destinationFile);

        slaveFile.delete();
      }
      catch (Throwable e) {
        System.out.println("error: " + e.toString());

        throw new IOException(e.getMessage());
      }
    }

    public void flush() throws IOException {
      slaveWriter.flush();
    }

    public void write(char cbuf[], int off, int len) throws IOException {
      slaveWriter.write(cbuf, off, len);
    }

    private Writer slaveWriter;
    private File slaveFile;
    private File destinationFile;
  }

}