/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package  mircoders.media;

import mir.entity.Entity;
import mir.log.LoggerWrapper;
import mir.media.MediaExc;
import mir.media.MediaFailure;
import mir.media.MediaHandler;
import mir.misc.StringUtil;
import mir.util.IORoutines;
import mircoders.global.MirGlobal;

import java.io.FileOutputStream;
import java.io.File;



/**
 * Handles realAudio .it manages the ram file.
 *
 * @author john <john@manifestor.org>, mh <heckmann@hbe.ca>
 * @version $Id: MediaHandlerRealAudio.java,v 1.19.2.8 2005/08/21 17:09:23 zapata Exp $
 */


public class MediaHandlerRealAudio extends MediaHandlerAudio implements MediaHandler
{
  public MediaHandlerRealAudio() {
    logger = new LoggerWrapper("Media.Audio.Real");
  }

  public void produce (Entity ent, Entity mediaTypeEnt ) throws MediaExc, MediaFailure {

    // first see if the file exists
    super.produce(ent, mediaTypeEnt);

    String date = ent.getFieldValue("date");
    String datePath = StringUtil.webdbDate2path(date);
    String rtspMediaHost = configuration.getString("Producer.RealMedia.Host");

    String RealMediaPointer = rtspMediaHost+ent.getFieldValue("publish_path");
    String RealMediaFile = datePath+ent.getId()+".ram";
    try {
      //write a ram file

      IORoutines.writeStream(RealMediaPointer, "us-ascii",
          new FileOutputStream(getBaseStoragePath() + File.separator + datePath + File.separator + RealMediaFile));
    }
    catch (Throwable e) {
      logger.error("Error while producing realmedia: " + e.toString(), e);

      throw new MediaFailure(e);
    }
  }

  public String getBaseStoragePath()
  {
    return configuration.getString("Producer.RealMedia.Path");
  }

  public String getDescr(Entity mediaType)
  {
    return "RealMedia";
  }

  public String getPublishHost()
  {
    return StringUtil.removeSlash(configuration.getString("Producer.RealMedia.Host"));
  }

}



