/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mircoders.servlet;

import mir.entity.adapter.EntityAdapter;
import mir.entity.adapter.EntityAdapterEngine;
import mir.entity.adapter.EntityAdapterModel;
import mir.generator.Generator;
import mir.misc.StringUtil;
import mir.servlet.AdminServletModule;
import mir.servlet.ServletModuleExc;
import mir.servlet.ServletModuleFailure;
import mir.util.HTTPRequestParser;
import mir.util.JDBCStringRoutines;
import mir.util.SQLQueryBuilder;
import mir.util.StringRoutines;
import mir.util.URLBuilder;
import mircoders.entity.EntityContent;
import mircoders.entity.EntityUsers;
import mircoders.global.MirGlobal;
import mircoders.module.ModuleContent;
import mircoders.storage.DatabaseContent;
import mircoders.storage.DatabaseContentToTopics;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Collections;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

/**
 * Article admin interface code
 */

public class ServletModuleContent extends AdminServletModule {
  private ModuleContent contentModule = new ModuleContent();

  public ServletModuleContent() {
    addPropagatedParameter("selectarticleurl");
    addPropagatedParameter("searchtopic");
    addPropagatedParameter("searchvalue");
    addPropagatedParameter("searcharticletype");

    definition = "content";
    mainModule = contentModule;
  }

  public void search(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc, ServletModuleFailure {
    try {
      String extraTables = "";
      HTTPRequestParser requestParser = new HTTPRequestParser(aRequest);
      SQLQueryBuilder queryBuilder = new SQLQueryBuilder();
      String searchField = requestParser.getParameterWithDefault("searchfield", "");
      String searchValue = requestParser.getParameterWithDefault("searchvalue", "").trim();
      String searchOrder = requestParser.getParameterWithDefault("searchorder", "");
      String searchispublished = requestParser.getParameterWithDefault("searchispublished", "");
      String searchArticleType = requestParser.getParameterWithDefault("searcharticletype", "");
      String searchTopic = requestParser.getParameterWithDefault("searchtopic", "");

      if (searchValue.length()>0) {
        if ("id".equals(searchField)) {
          queryBuilder.appendAndCondition(
            "id='"+JDBCStringRoutines.escapeStringLiteral(searchValue)+"'");
        }
        else if ("contents".equals(searchField)) {
          queryBuilder.appendAndCondition(
            "(lower(content_data) like " + "'%" + JDBCStringRoutines.escapeStringLiteral(searchValue.toLowerCase()) + "%')"+
            " or (lower(description) like " + "'%" + JDBCStringRoutines.escapeStringLiteral(searchValue.toLowerCase()) + "%')");
        }
        else {
          queryBuilder.appendAndCondition(
            "lower("+ searchField + ") like " +
            "'%" + JDBCStringRoutines.escapeStringLiteral(searchValue.toLowerCase()) + "%'");
        }
      }

      if (searchispublished.length()>0) {
        if ("0".equals(searchispublished)) {
          queryBuilder.appendAndCondition("is_published='f'");
        }
        else {
          queryBuilder.appendAndCondition("is_published='t'");
        }
      }

      if (searchArticleType.length()>0) {
        queryBuilder.appendAndCondition("to_article_type="+Integer.parseInt(searchArticleType));
      }

      if (searchOrder.length()>0) {
        if ("datedesc".equals(searchOrder)) {
          queryBuilder.appendDescendingOrder("webdb_create");
        }
        else if ("dateasc".equals(searchOrder)) {
          queryBuilder.appendAscendingOrder("webdb_create");
        }
        else if ("title".equals(searchOrder)) {
          queryBuilder.appendAscendingOrder("title");
        }
        else if ("creator".equals(searchOrder)) {
          queryBuilder.appendAscendingOrder("creator");
        }
      }

      if (searchTopic.length() > 0) {
        queryBuilder.appendAndCondition("cxt.content_id = id");
        queryBuilder.appendAndCondition("cxt.topic_id = " + JDBCStringRoutines.escapeStringLiteral(searchTopic));

        extraTables = "content_x_topic cxt";
      }

      returnList(aRequest, aResponse, queryBuilder.getWhereClause(), queryBuilder.getOrderByClause(), 0,
          Collections.EMPTY_MAP, null, extraTables);
    }
    catch (Throwable e) {
      throw new ServletModuleFailure(e);
    }
  }

  public void add(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    editObject(aRequest, aResponse, null);
  }

  public void insert(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    try {
      Map withValues = getIntersectingValues(aRequest, DatabaseContent.getInstance());

      String now = StringUtil.date2webdbDate(new GregorianCalendar());
      withValues.put("date", now);
      withValues.put("publish_path", StringUtil.webdbDate2path(now));
      withValues.put("to_publisher", ServletHelper.getUser(aRequest).getId());
      withValues.put("is_produced", "0");
      if (!withValues.containsKey("is_published")) {
        withValues.put("is_published","0");
      }
      if (!withValues.containsKey("is_html")) {
        withValues.put("is_html","0");
      }

      String webdbCreate = (String) withValues.get("webdb_create");
      if (webdbCreate==null || webdbCreate.trim().length()==0) {
        withValues.remove("webdb_create");
      }

      String id = mainModule.add(withValues);
      logAdminUsage(aRequest, id, "object added");

      DatabaseContentToTopics.getInstance().setTopics(id, aRequest.getParameterValues("to_topic"));

      editObject(aRequest, aResponse, id);
    }
    catch (Throwable e) {
      throw new ServletModuleFailure(e);
    }
  }

  public void edit(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    String idParam = aRequest.getParameter("id");
    if (idParam == null)
      throw new ServletModuleExc("Invalid call: id not supplied ");
    editObject(aRequest, aResponse, idParam);
  }

  /**
   * Attaches media to an article
   */
  public void attach(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    String  mediaIdParam = aRequest.getParameter("mid");
    String  articleId = aRequest.getParameter("articleid");
    if (articleId == null || mediaIdParam==null) {
      throw new ServletModuleExc("smod content :: attach :: articleid/mid missing");
    }

    // check if mediaIdParam and articleid are correct integers
    try {
      Integer.parseInt(mediaIdParam);
      Integer.parseInt(articleId);
    }
    catch (NumberFormatException e) {
      throw new ServletModuleExc("smod content :: attach :: invalid articleid/mid", e);
    }
    
    if (!MirGlobal.accessControl().article().mayEditArticle(ServletHelper.getUser(aRequest), articleId)) {
      throw new ServletModuleExc("Article has been locked");
    }

    try {
      EntityContent entContent = (EntityContent) mainModule.getById(articleId);
      entContent.attach(mediaIdParam);
    }
    catch(Throwable e) {
      throw new ServletModuleFailure(e);
    }

    logAdminUsage(aRequest, articleId, "media " + mediaIdParam + " attached");

    editObject(aRequest, aResponse, articleId);
  }

  /**
   * Deattaches media from an article
   */
  public void dettach(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    String  articleId = aRequest.getParameter("articleid");
    String  midParam = aRequest.getParameter("mid");
    if (articleId == null)
      throw new ServletModuleExc("smod content :: dettach :: articleid missing");
    if (midParam == null)
      throw new ServletModuleExc("smod content :: dettach :: mid missing");

    if (!MirGlobal.accessControl().article().mayEditArticle(ServletHelper.getUser(aRequest), articleId))
      throw new ServletModuleExc("Article has been locked");

    try {
      EntityContent entContent = (EntityContent)mainModule.getById(articleId);
      entContent.dettach(articleId, midParam);
    }
    catch(Throwable e) {
      throw new ServletModuleFailure(e);
    }

    logAdminUsage(aRequest, articleId, "media " + midParam + " deattached");

    editObject(aRequest, aResponse, articleId);
  }

  /**
   * Locks an article
   */
  public void lock(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc
  {
    HTTPRequestParser requestParser = new HTTPRequestParser(aRequest);

    String idParam = requestParser.getParameter("id");
    if (idParam == null) {
      throw new ServletModuleExc("Wrong call: (id) is missing");
    }

    EntityUsers user = ServletHelper.getUser(aRequest);

    if (!MirGlobal.accessControl().article().mayLockArticle(user, idParam)) {
      throw new ServletModuleExc("Unable to lock");
    }

    contentModule.lockArticle(idParam, user.getId(), false);

    editObject(aRequest, aResponse, idParam);
  }

  /**
   * Unlocks an article
   */
  public void unlock(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc
  {
    HTTPRequestParser requestParser = new HTTPRequestParser(aRequest);

    String idParam = requestParser.getParameter("id");
    if (idParam == null) {
      throw new ServletModuleExc("Wrong call: (id) is missing");
    }

    EntityUsers user = ServletHelper.getUser(aRequest);

    if (!MirGlobal.accessControl().article().mayUnlockArticle(user, idParam)) {
      throw new ServletModuleExc("Unable to unlock");
    }

    contentModule.unlockArticle(idParam, user.getId(), false);

    editObject(aRequest, aResponse, idParam);
  }

  /**
   * Forcelocks an article
   */
  public void forcelock(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    HTTPRequestParser requestParser = new HTTPRequestParser(aRequest);

    String idParam = requestParser.getParameter("id");
    if (idParam == null)
      throw new ServletModuleExc("Wrong call: (id) is missing");

    EntityUsers user = ServletHelper.getUser(aRequest);

    if (!MirGlobal.accessControl().article().mayForceLockArticle(user, idParam))
      throw new ServletModuleExc("Unable to force lock");

    contentModule.lockArticle(idParam, user.getId(), true);

    editObject(aRequest, aResponse, idParam);
  }

  /**
   * Stores an article
   */
  public void update(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    try {
      HTTPRequestParser requestParser = new HTTPRequestParser(aRequest);

      String idParam = requestParser.getParameter("id");
      if (idParam == null) {
        throw new ServletModuleExc("Wrong call: (id) is missing");
      }

      if (!MirGlobal.accessControl().article().mayEditArticle(ServletHelper.getUser(aRequest), idParam)) {
        throw new ServletModuleExc("Article has been locked");
      }

      Map withValues = getIntersectingValues(aRequest, DatabaseContent.getInstance());

      withValues.put("is_produced", "0");
      if (!withValues.containsKey("is_published")) {
        withValues.put("is_published","0");
      }
      if (!withValues.containsKey("is_html")) {
        withValues.put("is_html","0");
      }

      String webdbCreate = (String) withValues.get("webdb_create");
      if (webdbCreate==null || webdbCreate.trim().length()==0) {
        withValues.remove("webdb_create");
      }

      String id = mainModule.set(withValues);

      logAdminUsage(aRequest, id, "object modified");

      DatabaseContentToTopics.getInstance().setTopics(aRequest.getParameter("id"), aRequest.getParameterValues("to_topic"));

      if (MirGlobal.accessControl().article().mayUnlockArticle(ServletHelper.getUser(aRequest), idParam) &&
          requestParser.getParameterWithDefault("unlock", "0").equals("1")) {
        contentModule.unlockArticle(id, ServletHelper.getUser(aRequest).getId(), false);
      }

      editObject(aRequest, aResponse, idParam);
    }
    catch (Throwable e) {
      throw new ServletModuleFailure(e);
    }
  }


  /**
   * Returns the basic article editing form.
   *
   * @param anId identifier of the article. <code>null</code>, means show an
   *     empty form to add a new article.
   */
  public void editObject(HttpServletRequest aRequest, HttpServletResponse aResponse,
                         String anId) throws ServletModuleExc {
    try {
      HTTPRequestParser requestParser = new HTTPRequestParser(aRequest);
      Map responseData = ServletHelper.makeGenerationData(aRequest, aResponse, new Locale[] { getLocale(aRequest), getFallbackLocale(aRequest)});
      EntityAdapterModel model = MirGlobal.localizer().dataModel().adapterModel();
      Object article;
      URLBuilder urlBuilder = new URLBuilder();

      urlBuilder.setValue("module", "Content");
      urlBuilder.setValue("do", "edit");
      urlBuilder.setValue("id", anId);
      urlBuilder.setValue("returnurl", requestParser.getParameter("returnurl"));

      if (anId !=null) {
        responseData.put("new", Boolean.FALSE);
        article = model.makeEntityAdapter("content", mainModule.getById(anId));

        EntityUsers user = ServletHelper.getUser(aRequest);

        responseData.put("mayEdit",
            new Boolean(MirGlobal.accessControl().article().mayEditArticle(user, anId)));
        responseData.put("mayLock", new Boolean(MirGlobal.accessControl().article().mayLockArticle(user, anId)));
        responseData.put("mayForceLock", new Boolean(MirGlobal.accessControl().article().mayForceLockArticle(user, anId)));
        responseData.put("mayUnlock", new Boolean(MirGlobal.accessControl().article().mayUnlockArticle(user, anId)));
      }
      else {
        List fields = DatabaseContent.getInstance().getFieldNames();
        responseData.put("new", Boolean.TRUE);
        Map emptyArticle = new HashMap();
        Iterator i = fields.iterator();
        while (i.hasNext()) {
          emptyArticle.put(i.next(), null);
        }

        emptyArticle.put("to_topics", null);

        MirGlobal.localizer().adminInterface().initializeArticle(emptyArticle);
        article = emptyArticle;

        responseData.put("mayEdit", Boolean.TRUE);
        responseData.put("mayLock", Boolean.FALSE);
        responseData.put("mayForceLock", Boolean.FALSE);
        responseData.put("mayUnlock", Boolean.FALSE);

      }
      responseData.put("article", article);

      List topicsList = new ArrayList();

      String[] topicCategories = getConfiguration().getStringArray("Mir.Localizer.Admin.TopicLists");

      if (topicCategories.length==0 ) {
        Map categoryMap = new HashMap();
        categoryMap.put("key", "topic");
        categoryMap.put("listtype", "0");
        categoryMap.put("listparameter", "3");
        categoryMap.put("items", EntityAdapterEngine.retrieveAdapterList(model, "topic", "", "title", -1, 0));
        topicsList.add(categoryMap);
      }
      else {
        for (int i = 0; i < topicCategories.length; i++) {
          Map categoryMap = new HashMap();
          List parts = StringRoutines.splitString(topicCategories[i], ":");

          String key = null;

          if (parts.size() > 0) {
            key = (String) parts.get(0);
          }
          String listtype = "0";
          if (parts.size() > 1) {
            listtype = (String) parts.get(1);
          }
          String listparameter = "5";
          if (parts.size() > 2) {
            listparameter = (String) parts.get(2);
          }
          String where = "";
          if (parts.size() > 3) {
            where = (String) parts.get(3);
          }
          String order = "";
          if (parts.size() > 4) {
            order = (String) parts.get(4);
          }

          if (key != null) {
            categoryMap.put("key", key);
            categoryMap.put("listtype", listtype);
            categoryMap.put("listparameter", listparameter);
            categoryMap.put("items", EntityAdapterEngine.retrieveAdapterList(model, "topic", where, order, -1, 0));
            topicsList.add(categoryMap);
          }
        }
      }

      responseData.put("topics", topicsList);

      responseData.put("returnurl", requestParser.getParameter("returnurl"));
      responseData.put("thisurl", urlBuilder.getQuery());

      ServletHelper.generateResponse(aResponse.getWriter(), responseData, editGenerator);
    }
    catch (Throwable e) {
      throw new ServletModuleFailure(e);
    }
  }

  public void selectparent(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    try {
      HTTPRequestParser requestParser = new HTTPRequestParser(aRequest);
      URLBuilder urlBuilder = new URLBuilder();

      urlBuilder.setValue("module", "Content");
      urlBuilder.setValue("do", "setparent");
      urlBuilder.setValue("childid", requestParser.getParameter("id"));
      urlBuilder.setValue("returnurl", requestParser.getParameter("returnurl"));

      returnList(aRequest, aResponse, "", "", 0,
          Collections.singletonMap("selectarticleurl", urlBuilder.getQuery()));
    }
    catch (Throwable e) {
      throw new ServletModuleFailure(e);
    }
  }

  public void listchildren(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    HTTPRequestParser requestParser = new HTTPRequestParser(aRequest);
    String articleId = requestParser.getParameter("article_id");

    if (articleId == null) {
      throw new ServletModuleExc("ServletModuleContent.listchildren: article_id not set!");
    }

    try {
      returnList(aRequest, aResponse, "to_content = " + articleId, "webdb_create desc", 0);
    }
    catch (Throwable e) {
      throw new ServletModuleFailure(e);
    }
  }

  public void setparent(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    HTTPRequestParser requestParser = new HTTPRequestParser(aRequest);
    String articleId = requestParser.getParameter("childid");
    String parentId  = requestParser.getParameter("id");
    String returnUrl = requestParser.getParameter("returnurl");

    if (!MirGlobal.accessControl().article().mayEditArticle(ServletHelper.getUser(aRequest), articleId))
      throw new ServletModuleExc("Article has been locked");

    try {
      EntityContent article = (EntityContent) mainModule.getById(articleId);
      article.setFieldValue("to_content", parentId);
      article.setProduced(false);
      article.update();
      logAdminUsage(aRequest, articleId, "parent set to " + parentId);
    }
    catch(Throwable e) {
      getLogger().error("ServletModuleContent.setparent: " + e.getMessage());

      throw new ServletModuleFailure(e);
    }

    ServletHelper.redirect(aResponse, returnUrl);
  }

  public void clearparent(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    HTTPRequestParser requestParser = new HTTPRequestParser(aRequest);
    String articleId = requestParser.getParameter("id");
    String returnUrl = requestParser.getParameter("returnurl");

    try {
      EntityContent article = (EntityContent) mainModule.getById(articleId);
      article.setFieldValue("to_content", null);
      article.setProduced(false);
      article.update();
      logAdminUsage(aRequest, articleId, "parent cleared");
    }
    catch(Throwable e) {
      getLogger().error("ServletModuleContent.clearparent: " + e.getMessage(), e);

      throw new ServletModuleFailure("ServletModuleContent.clearparent: " + e.getMessage(), e);
    }

    ServletHelper.redirect(aResponse, returnUrl);
  }

  public void showPreview(HttpServletRequest aRequest, HttpServletResponse aResponse) throws ServletModuleExc {
    try {
      HTTPRequestParser requestParser = new HTTPRequestParser(aRequest);
      String articleId = requestParser.getParameter("id");
      EntityAdapter article = getModel().makeEntityAdapter("content", mainModule.getById(articleId));
      String preview = requestParser.getParameterWithDefault("preview", "default");

      Map generationValues = new HashMap();
      Generator generator =
          MirGlobal.localizer().adminInterface().prepareArticlePreview(preview, article, generationValues);

      generator.generate(aResponse.getWriter(), generationValues, getLogger());
    }
    catch (Exception e) {
      throw new ServletModuleFailure(e);
    }
  }
}
